@@ -4,11 +4,10 @@ import pathlib
 
 import numpy as np
 import yaml
-
 from pyrosenv import rosbag, rospy
 from pyrosenv import genpy
 
-TimeLike = Union[float, rospy.rostime.Time]
+TimeLike = Union[float, rospy.Time, genpy.Time]
 PathLike = Union[str, pathlib.Path]
 RosMessage = genpy.Message
 
@@ -26,10 +25,15 @@ In this module there are two types of time for each msg:
 """
 
 
+class BagManagerException(Exception):
+    pass
+
+
 class BagManager:
     def __init__(self, bag_file: PathLike):
         self.bag = rosbag.Bag(bag_file)
         self.bag_info = self._get_bag_info(bag_file)
+        self.topics_info_cache = {}
 
     @staticmethod
     def _get_bag_info(bag_file: PathLike) -> Dict:
@@ -50,12 +54,17 @@ class BagManager:
         entry_gen = self.bag._get_entries(connections, start_time=None, end_time=None)
         msg_time_list_rosbag = [entry.time for entry in entry_gen]
         if get_header_time:
-            try:
-                msg_time_list_header = [msg.header.stamp for _, msg, _ in self.bag.read_messages(topics=[topic])]
-            except AttributeError:
-                msg_time_list_header = "Some msg doesn't have header timestamp"
+            if topic in self.topics_info_cache:
+                msg_time_list_header = self.topics_info_cache[topic]
+            else:
+                try:
+                    msg_time_list_header = [msg.header.stamp for _, msg, _ in self.bag.read_messages(topics=[topic])]
+                except AttributeError:
+                    msg_time_list_header = BagManagerException("Some msg doesn't have header timestamp")
+                self.topics_info_cache[topic] = msg_time_list_header
+
         else:
-            msg_time_list_header = "Call get_topic_info() with get_header_time=True to get the stamps from the headers"
+            msg_time_list_header = BagManagerException("Call get_topic_info() with get_header_time=True to get the headers stamps")
         topic_info = {'topic': topic, 'message_count': topic_tuple.message_count,
                       'message_type': topic_tuple.msg_type, 'frequency': topic_tuple.frequency,
                       'msg_time_list_header': msg_time_list_header, 'msg_time_list_rosbag': msg_time_list_rosbag}
@@ -63,8 +72,8 @@ class BagManager:
 
     def get_closest_message_by_header_time(self, topic: str, time_header: TimeLike) -> RosMessage:
         """ Returns a message from the given topic with header timestamp closest to time_header """
-        if not isinstance(time_header, rospy.rostime.Time):
-            time_header = rospy.rostime.Time(time_header)
+        if not isinstance(time_header, rospy.Time) and not isinstance(time_header, genpy.Time):
+            time_header = rospy.Time(time_header)
         info = self.get_topic_info(topic=topic, get_header_time=True)
         argmin = np.argmin([abs(t - time_header) for t in info['msg_time_list_header']])
         matching_msg_time_rosbag = info['msg_time_list_rosbag'][argmin]
@@ -75,8 +84,8 @@ class BagManager:
 
     def get_closest_message_by_rosbag_time(self, topic: str, time_rosbag: TimeLike) -> RosMessage:
         """ Returns a message from the given topic with rosbag timestamp closest to time_rosbag """
-        if not isinstance(time_rosbag, rospy.rostime.Time):
-            time_rosbag = rospy.rostime.Time(time_rosbag)
+        if not isinstance(time_rosbag, rospy.Time) and not isinstance(time_rosbag, genpy.Time):
+            time_rosbag = rospy.Time(time_rosbag)
         info = self.get_topic_info(topic=topic, get_header_time=False)
         argmin = np.argmin([abs(t - time_rosbag) for t in info['msg_time_list_rosbag']])
         matching_msg_time_rosbag = info['msg_time_list_rosbag'][argmin]
@@ -112,17 +121,17 @@ class BagManager:
         if end_time_rosbag is None:
             end_time_rosbag = self.bag_info['end']
 
-        if not isinstance(start_time_rosbag, rospy.rostime.Time):
-            start_time_rosbag = rospy.rostime.Time(start_time_rosbag)
-        if not isinstance(end_time_rosbag, rospy.rostime.Time):
-            end_time_rosbag = rospy.rostime.Time(end_time_rosbag)
+        if not isinstance(start_time_rosbag, rospy.Time) and not isinstance(start_time_rosbag, genpy.Time):
+            start_time_rosbag = rospy.Time(start_time_rosbag)
+        if not isinstance(end_time_rosbag, rospy.Time) and not isinstance(end_time_rosbag, genpy.Time):
+            end_time_rosbag = rospy.Time(end_time_rosbag)
 
         message_count = 0
         for topic in topics:
             msg_list = self.get_topic_info(topic=topic, get_header_time=False)['msg_time_list_rosbag']
             number_of_msgs_in_interval = (np.searchsorted(msg_list, end_time_rosbag, side='right')
                                           - np.searchsorted(msg_list, start_time_rosbag, side='left'))
-
+                                          
             message_count += number_of_msgs_in_interval
         return message_count
 
